# begin_generated_IBM_copyright_prolog
#
# Licensed Materials - Property of IBM
# 5724-N72 5655-WBS
# (c) Copyright IBM Corp. 2008, 2014 All Rights Reserved
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#
# end_generated_IBM_copyright_prolog

import sys
import os

# Simple routine sets calling routine variables from command line parameters
# Parameters:
#   argv -- sequence of parameters (from command line)
#	options -- array of [option, variable name] pairs
#              e.g., [ ["-u", "user"], ["-p", "password"] ]
#   varDict -- dictionary in which to set variable associated with option
def processArgs(argv, options, varDict) :	
	programName = argv[0]
	for opt in options :
		varDict[opt[1]] = None	# initialize variables specified in options
	_processArgs(programName, argv[1:], options, varDict)
	for opt in options :
		if varDict[opt[1]] == None:
			if opt[2] == "true":
				usage(programName, "missing option " + opt[0], options)
			else:
				varDict[opt[1]] = "null"

# Auxilliary routine processes next two command line parameters,
# 	then calls itself to process the rest of the command line
def _processArgs(programName, argv, options, varDict) :
	if len(argv) == 0 :
		return

	option = argv[0]
	if option == "-?" or option == "-h" or option == "--help" :
		usage(programName, None, options)

	if len(argv) <= 1 :
		usage(programName, "illegal argument count", options)
	value = argv[1]

	for opt in options :
		if option == opt[0] :
			varDict[opt[1]] = value	# set variable in dictionary as specified in options
			break
	else :
		usage(programName, "unknown option " + option, options)

	_processArgs(programName, argv[2:], options, varDict)

# Usage message, constructed from command line options
def usage(programName, message, options) :
	if message != None :
		print "Error exit: " + message
	usage = "usage: jython " + programName
	for opt in options :
		if opt[2] == "true":
			usage += " " + opt[0] + " <" + opt[1] + ">"
		else:
			usage += " [" + opt[0] + " <" + opt[1] + ">]"

	print usage
	sys.exit(1)


True=1==1
False=1==0

class Module:
    def __init__(self,name):
        self.name = name
        self.uri = ""
        self.oldMapping = ""

    def getMapping(self,webServer):
        print "Setting mapping for module "+self.name+" to "+webServer+"+"+self.oldMapping
        return webServer+"+"+self.oldMapping
    

def checkIfAppExists(appName):
    application=AdminConfig.getid("/Deployment/"+appName+"/")
    return len(application) != 0

def mapToServers(webServer,appName):
    modulesToServer=AdminApp.view(appName,["-MapModulesToServers"]).replace("\r\n","\n")
    modulesToServer=modulesToServer.split("\n")
    mappings=[]
    currentModule=None
    for line in modulesToServer:
        if line.startswith("Module:"):
            if(currentModule != None):
                print "Never found a URI/Server for module "+currentModule.name+" - discarding it"
            currentModule = Module(line[len("Module:  "):])
        elif line.startswith("URI:"):
            if(currentModule == None):
                print "Found a URI without a module "+line+" - ignoring it"
            else:
                currentModule.uri = line[len("URI:  "):]
        elif line.startswith("Server:"):
            if(currentModule == None):
                print "Found a Server without a module "+line+" - ignoring it"
            else:
                currentModule.oldMapping = line[len("Server: "):].strip()
                print "Found module "+currentModule.name+" uri "+currentModule.uri+" mapped to "+currentModule.oldMapping
                mappings.append(currentModule)
                currentModule = None
        elif line.strip() == "":            
            pass
        else:
            print "Ignoring line: "+line

    mappings = map(lambda x,w=webServer: [x.name,x.uri,x.getMapping(w)], mappings)

    print "Applying mappings"
    AdminApp.edit(appName,["-MapModulesToServers",mappings])

    print "Saving changes"
    AdminConfig.save()

    print "Sync nodes"
    nodeSyncObjects = AdminControl.queryNames("type=NodeSync,*")
    if len(nodeSyncObjects) > 0:
        for nodeSync in nodeSyncObjects.split(lineSeparator):
            try:
                print "Syncing "+str(nodeSync)
                syncResult = AdminControl.invoke(nodeSync, "sync", "")
                print "- returned "+str(syncResult)
            except:
                print "- threw exception"
    return True

def _splitlines(s):
    rv = [s]
    if '\r' in s:
        rv = s.split('\r\n')
    elif '\n' in s:
        rv = s.split('\n')
    if rv[-1] == '':
        rv = rv[:-1]
    return rv

def getDmgrNode():
    """Return config id of the Dmgr node"""
    node_ids = _splitlines(AdminConfig.list( 'Node' ))
    for node_id in node_ids:
        nodename = getNodeName(node_id)
        if nodeIsDmgr(nodename):
            return node_id
    return None

def getDmgrNodeName():
    """Return node name of the Dmgr node"""
    return getNodeName(getDmgrNode())
    
def getNodeName(node_id):
    """Get the name of the node with the given config object ID"""
    return AdminConfig.showAttribute(node_id, "name")
    
def nodeIsIHS( nodename ):
    """Returns true if the node is IHS."""
    # Note: This method queries whether variable WAS_INSTALL_ROOT is defined.
    # This is a weak technique for identifying an IHS node.
    # Hopefully a more robust mechanism can be found in the future.
    return None == getWasInstallRoot(nodename)

def getNodeId( nodename ):
    """Given a node name, get its config ID"""
    return AdminConfig.getid( '/Cell:%s/Node:%s/' % ( AdminControl.getCell(), nodename ) )

def nodeIsDmgr( nodename ):
    """Return true if the node is the deployment manager"""
    return nodeHasServerOfType( nodename, 'DEPLOYMENT_MANAGER' )

def nodeHasServerOfType( nodename, servertype ):
    node_id = getNodeId(nodename)
    serverEntries = _splitlines(AdminConfig.list( 'ServerEntry', node_id ))
    for serverEntry in serverEntries:
        sType = AdminConfig.showAttribute( serverEntry, "serverType" )
        if sType == servertype:
            return 1
    return 0
 
def getNodeVariable(nodename, varname):
    """Return the value of a variable for the node -- or None if no such variable or not set"""
    vmaps = _splitlines(AdminConfig.list('VariableMap', getNodeId(nodename)))
    if 0 < len(vmaps):  # Tolerate nodes with no such maps, for example, IHS nodes.
        map_id = vmaps[-1] # get last one
        entries = AdminConfig.showAttribute(map_id, 'entries')
        # this is a string '[(entry) (entry)]'
        entries = entries[1:-1].split(' ')
        for e in entries:
            name = AdminConfig.showAttribute(e,'symbolicName')
            value = AdminConfig.showAttribute(e,'value')
            if name == varname:
                return value
    return None

def getWasInstallRoot(nodename):
    """Return the absolute path of the given node's WebSphere installation"""
    return getNodeVariable(nodename, "WAS_INSTALL_ROOT")

def getWasProfileRoot(nodename):
    """Return the absolute path of the given node's profile directory"""
    return getNodeVariable(nodename, "USER_INSTALL_ROOT")

 
def listAppServerNodes():
    """Returns a list of nodes excluding dmgr and IHS nodes"""
    m = "listAppServerNodes:"
    node_ids = _splitlines(AdminConfig.list( 'Node' ))
    result = []
    for node_id in node_ids:
        nodename = AdminConfig.showAttribute(node_id, "name")
        if not nodeIsDmgr(nodename) and not nodeIsIHS(nodename):
            result.append(nodename)
    if 0 == len(result):
        sop(m,"Warning. No non-manager/non-IHS nodes are defined!!!")
    return result

#---------------------------------------------------------------------------------------
# 
# Main
#
#---------------------------------------------------------------------------------------

options = [
	[ "-cluster", "clusterName", "true" ],
	[ "-webNodeName", "webNodeName", "true" ],
	[ "-webServerName", "webServerName", "true" ],
	[ "-prefix", "prefix", "false" ]
]

## process command line arguments
sys.argv.insert(0,"configureIHS.py")
processArgs(sys.argv, options, globals())
cellName = AdminControl.getCell()
#get list of cluster members
clusterID = AdminConfig.getid("/ServerCluster:"+clusterName+"/" )
temp = AdminConfig.showAttribute(clusterID, "members" )
temp = temp.replace('[','')
temp = temp.replace(']','')
memberList = temp.strip().split(" ")

webServerId = AdminConfig.getid('/Node:'+webNodeName+'/Server:'+webServerName+'/')
webNodeId = AdminConfig.getid('/Node:'+webNodeName+'/')
webServerHost=AdminConfig.showAttribute(AdminConfig.list("ServerIndex", webNodeId),"hostName")
version=AdminTask.getNodeBaseProductVersion('[-nodeName '+webNodeName+']')
mapping = AdminConfig.getObjectName(webServerId)
dmgrNodeName = getDmgrNodeName()
configDir = getWasProfileRoot(dmgrNodeName)+'/config'

if prefix == None
	prefix = ""

print " -----------------------"
print " -- mapping modules."
print " -----------------------"
apps = ['ServiceRegistry', 'WSRRReportViewer', 'WSRRCRE']
for appName in apps:
	appName = prefix+appName
    if checkIfAppExists(appName):
        if mapToServers(mapping,appName):
            print "Modules mapped successfully for "+appName
        else:
            print "Map modules failed for "+appName
    else:
        print "Application "+appName+" not installed so mapping unchanged"

print " -----------------------"
print " -- module mapping completed."
print " -----------------------"

if os.path.isfile('/tmp/key'):
    os.remove('/tmp/key')

print " -----------------------"
print " -- Extracting certificate from cell"
print " -----------------------" 
AdminTask.extractSignerCertificate('[ -keyStoreName CellDefaultTrustStore -keyStoreScope (cell):' +cellName+' -certificateFilePath /tmp/key -base64Encoded true -certificateAlias root ]')

print " -----------------------"
print " -- Importing Certificate from cell to webserver"
print " -----------------------"
AdminTask.addSignerCertificate('[-keyStoreName CMSKeyStore -keyStoreScope (cell):'+cellName+':(node):'+webNodeName+':(server):'+webServerName+' -certificateFilePath /tmp/key -base64Encoded true -certificateAlias CELL' +cellName+' ]')
os.remove('/tmp/key')

for node in listAppServerNodes():
    print " -----------------------"
    print " -- Extracting certificate from %s " % (node)
    print " -----------------------"  
    AdminTask.extractSignerCertificate('[ -keyStoreName NodeDefaultTrustStore -keyStoreScope (cell):'+cellName+':(node):'+node+' -certificateFilePath /tmp/key -base64Encoded true -certificateAlias default ]')
    print " -----------------------"
    print " -- Importing Certificate from %s to webserver" % (node)
    print " -----------------------"
    AdminTask.addSignerCertificate('[-keyStoreName CMSKeyStore -keyStoreScope (cell):'+cellName+':(node):'+webNodeName+':(server):'+webServerName+' -certificateFilePath /tmp/key -base64Encoded true -certificateAlias NODE' +node+' ]')
    AdminConfig.save()
    os.remove('/tmp/key')
#endfor

AdminConfig.save()

print " -----------------------"
print " -- Pushing the keystore to the webserver"
print " -----------------------"
AdminControl.invoke("WebSphere:name=PluginCfgGenerator,process=dmgr,platform=common,node=%s,version=%s,type=PluginCfgGenerator,mbeanIdentifier=PluginCfgGenerator,cell=%s,spec=1.0" % (dmgrNodeName,version,cellName), "propagateKeyring", "[%s %s %s %s]" % (configDir, cellName , webNodeName, webServerName), "[java.lang.String java.lang.String java.lang.String java.lang.String]")
AdminConfig.save()  
 
print " -----------------------"
print " -- Syncing nodes"
print " -----------------------"
nodeSyncObjects = AdminControl.queryNames("type=NodeSync,*")
if len(nodeSyncObjects) > 0:
    for nodeSync in nodeSyncObjects.split(lineSeparator):
        try:
            print "Syncing "+str(nodeSync)
            syncResult = AdminControl.invoke(nodeSync, "sync", "")
            print "- returned "+str(syncResult)
        except:
            print "- threw exception"

print " -----------------------"
print " -- Restarting webserver"
print " -----------------------"
AdminControl.invoke("WebSphere:name=WebServer,process=dmgr,platform=common,node=%s,version=%s,type=WebServer,mbeanIdentifier=WebServer,cell=%s,spec=1.0" % (dmgrNodeName,version,cellName), 'stop', "[ %s %s %s ]" % (cellName , webNodeName, webServerName), "[java.lang.String java.lang.String java.lang.String]")
AdminControl.invoke("WebSphere:name=WebServer,process=dmgr,platform=common,node=%s,version=%s,type=WebServer,mbeanIdentifier=WebServer,cell=%s,spec=1.0" % (dmgrNodeName,version,cellName), 'ping', "[ %s %s %s ]" % (cellName , webNodeName, webServerName), "[java.lang.String java.lang.String java.lang.String]")
AdminControl.invoke("WebSphere:name=WebServer,process=dmgr,platform=common,node=%s,version=%s,type=WebServer,mbeanIdentifier=WebServer,cell=%s,spec=1.0" % (dmgrNodeName,version,cellName), 'start',"[ %s %s %s ]" % (cellName , webNodeName, webServerName), "[java.lang.String java.lang.String java.lang.String]")
AdminControl.invoke("WebSphere:name=WebServer,process=dmgr,platform=common,node=%s,version=%s,type=WebServer,mbeanIdentifier=WebServer,cell=%s,spec=1.0" % (dmgrNodeName,version,cellName), 'ping', "[ %s %s %s ]" % (cellName , webNodeName, webServerName), "[java.lang.String java.lang.String java.lang.String]") 	
print " -----------------------"
print " -- Complete."
print " -----------------------"
